﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace PrintDriverIntegration
{
    public partial class Form1 : Form
    {
        private readonly SlidePrinter _slidePrinter;
        private string _currentImage;
        public Form1()
        {
            InitializeComponent();
            _currentImage = "";
            _slidePrinter = SlidePrinter.Instance;
            _slidePrinter.SetPrinter("Slide Printer");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            buttonAutoRefresh.Text = @"Start Auto Refresh";
            UpdateImageWindow("");
            UpdateSlideStatusWindow();
        }

        private void buttonAutoRefresh_Click(object sender, EventArgs e)
        {
            if (timerAutoRefresh.Enabled)
            {
                timerAutoRefresh.Stop();
                buttonAutoRefresh.Text = @"Start Auto Refresh";
                buttonRefresh.Enabled = true;
            }
            else
            {
                timerAutoRefresh.Start();
                buttonAutoRefresh.Text = @"Stop Auto Refresh";
                buttonRefresh.Enabled = false;
            }
        }
        private void UpdateSlideStatusWindow()
        {
            _slidePrinter.Refresh(2, 10);
            label1.Text = _slidePrinter.ToString();


        }

        private void buttonRefresh_Click(object sender, EventArgs e)
        {
            UpdateSlideStatusWindow();
        }

        private void buttonLoadImage_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog dlg = new OpenFileDialog())
            {
                dlg.Title = @"Open Image";
                dlg.Filter = @"bmp files (*.bmp)|*.bmp";

                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    UpdateImageWindow(dlg.FileName);


                }
            }
        }

        private void UpdateImageWindow(string file)
        {
            if (String.IsNullOrEmpty(file))
            {
                labelCurrentImage.Text = @"Current Image: None (Drag a BMP here)";
                pictureBox1.Image = null;
                _currentImage = "";
                buttonPrintImage.Enabled = false;
            }
            else
            {
                pictureBox1.Image = new Bitmap(file);
                pictureBox1.SizeMode = PictureBoxSizeMode.CenterImage;
                _currentImage = file;
                labelCurrentImage.Text = @"Current Image: " + Path.GetFileName(file);
                buttonPrintImage.Enabled = true;
            }

        }

        private void buttonClearImage_Click(object sender, EventArgs e)
        {
            UpdateImageWindow("");
        }

        private void buttonPrintImage_Click(object sender, EventArgs e)
        {
            try
            {
                _slidePrinter.PrintImageFile(_currentImage);
            }
            catch (Exception ex)
            {
                MessageBox.Show(@"Exception Printing Current Image " + ex.Message);
            }

        }

        private void timerAutoRefresh_Tick(object sender, EventArgs e)
        {
            UpdateSlideStatusWindow();
        }


        private void Form1_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

            FileInfo firstFile = new FileInfo(files[0]);
            if (firstFile.Exists)
                UpdateImageWindow(firstFile.FullName);
        }

        private void buttonResetSlideCount_Click(object sender, EventArgs e)
        {
            if (!_slidePrinter.SetSlideCount(100))
            {
                MessageBox.Show(@"Error Setting Slide Count");
            }
        }
    }
}
