﻿// ********************************************************************************
// PrintLabel.cs
// 04/2012
// Primera Technology, Inc.
// ********************************************************************************

using System;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Runtime.InteropServices;
using System.Threading;

namespace PrintDriverIntegration
{
    class PrintLabel
    {
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        private class DOCINFOA
        {
            [MarshalAs(UnmanagedType.LPStr)]
            public string pDocName;
            [MarshalAs(UnmanagedType.LPStr)]
            public string pOutputFile;
            [MarshalAs(UnmanagedType.LPStr)]
            public string pDataType;
        }
        [DllImport("winspool.Drv", EntryPoint = "OpenPrinterA", SetLastError = true, CharSet = CharSet.Ansi, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool OpenPrinter([MarshalAs(UnmanagedType.LPStr)] string szPrinter, out IntPtr hPrinter, ref UInt32 pd);

        [DllImport("winspool.Drv", EntryPoint = "ClosePrinter", SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool ClosePrinter(IntPtr hPrinter);

        [DllImport("winspool.Drv", EntryPoint = "StartDocPrinterA", SetLastError = true, CharSet = CharSet.Ansi, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool StartDocPrinter(IntPtr hPrinter, Int32 level, [In, MarshalAs(UnmanagedType.LPStruct)] DOCINFOA di);

        [DllImport("winspool.Drv", EntryPoint = "EndDocPrinter", SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool EndDocPrinter(IntPtr hPrinter);

        [DllImport("winspool.Drv", EntryPoint = "StartPagePrinter", SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool StartPagePrinter(IntPtr hPrinter);

        [DllImport("winspool.Drv", EntryPoint = "EndPagePrinter", SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool EndPagePrinter(IntPtr hPrinter);

        [DllImport("winspool.Drv", EntryPoint = "WritePrinter", SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        private static extern bool WritePrinter(IntPtr hPrinter, IntPtr pBytes, Int32 dwCount, out Int32 dwWritten);

        [DllImport("winspool.Drv", EntryPoint = "SetPrinterData")]
        private static extern int SetPrinterData(IntPtr hPrinter, [MarshalAs(UnmanagedType.LPStr)] string pValueName, int dwType, ref byte pData, int cbData);

        //[DllImport("winspool.drv", CharSet=CharSet.Auto)]
        //static extern uint SetPrinterData(IntPtr hPrinter, string pValueName, uint Type, byte[] pData, uint cbData);


        [DllImport("winspool.Drv", EntryPoint = "GetPrinterData")]
        private static extern int GetPrinterData(IntPtr hPrinter, [MarshalAs(UnmanagedType.LPStr)] string pValueName, ref int pType, ref byte pData, int nSize, ref int pcbNeeded);


        public string Printer { get; set; }
        public string File { private get; set; }

        public bool ReadBytesFromPrinter(string szPrinterName, Byte[] bytes, Int32 dwCount, ref Int32  dwBytes, string sKey)
        {
            bool bSuccess; // Assume failure unless you specifically succeed.
            Int32 type = 3;
            byte[] bData = new byte[2];
            string sValue = sKey;
            const int numBytes = 1;
            const int setType = 3;

            bData[0] = 0;
            bData[1] = 0;
            int nRetries = 2;

            while (true)
            {
                // Open the printer.
                UInt32[] printDefaults = new UInt32[3];
                printDefaults[2] = 0x04;

                IntPtr hPrinter;
                if (OpenPrinter(szPrinterName, out hPrinter, ref printDefaults[0]))
                {
                    if (SetPrinterData(hPrinter, sValue, setType, ref bData[0], numBytes) == 0)
                    {
                        dwBytes = 0;
                        if (GetPrinterData(hPrinter, sValue, ref type, ref bytes[0], dwCount, ref dwBytes) == 0)
                        {

                            bSuccess = true;
                            ClosePrinter(hPrinter);
                            break;
                        }
                    }
                    Thread.Sleep(500);
                    ClosePrinter(hPrinter);
                }
                Thread.Sleep(500);
                if (nRetries-- <= 0)
                {
                    bSuccess = false;
                    break;
                }
            }

            return bSuccess;
        }

        public bool PrintRawFile()
        {
            // Open the file.
            FileStream fs = new FileStream(File, FileMode.Open, FileAccess.Read);
            // Create a BinaryReader on the file.
            BinaryReader br = new BinaryReader(fs);
            // Dim an array of bytes big enough to hold the file's contents.
            // Your unmanaged pointer.

            int nLength = Convert.ToInt32(fs.Length);
            // Read the contents of the file into the array.
            byte[] bytes = br.ReadBytes(nLength);
            // Allocate some unmanaged memory for those bytes.
            IntPtr pUnmanagedBytes = Marshal.AllocCoTaskMem(nLength);
            // Copy the managed byte array into the unmanaged array.
            Marshal.Copy(bytes, 0, pUnmanagedBytes, nLength);
            // Send the unmanaged bytes to the printer.
            bool bSuccess = SendBytesToPrinter(Printer, pUnmanagedBytes, nLength);
            // Free the unmanaged memory that you allocated earlier.
            Marshal.FreeCoTaskMem(pUnmanagedBytes);
            fs.Close();
            return bSuccess;

        }

        public bool SendBytesToPrinter(string szPrinterName, IntPtr pBytes, Int32 dwCount)
        {
            IntPtr hPrinter;
            DOCINFOA di = new DOCINFOA();
            bool bSuccess = false; // Assume failure unless you specifically succeed.

            di.pDocName = "Driver";
            di.pDataType = "RAW";

            // Open the printer.

            UInt32[] printDefaults = new UInt32[3];
            printDefaults[2] = 0x04;

            if (OpenPrinter(szPrinterName, out hPrinter, ref printDefaults[0]))
            {
                // Start a document.
                if (StartDocPrinter(hPrinter, 1, di))
                {
                    // Start a page.
                    if (StartPagePrinter(hPrinter))
                    {
                        // Write your bytes.
                        Int32 dwWritten;
                        bSuccess = WritePrinter(hPrinter, pBytes, dwCount, out dwWritten);
                        EndPagePrinter(hPrinter);
                    }
                    EndDocPrinter(hPrinter);
                }
                ClosePrinter(hPrinter);
            }
            // If you did not succeed, GetLastError may give more information
            // about why not.
            if (bSuccess == false)
            {
                Marshal.GetLastWin32Error();
            }
            return bSuccess;
        }
        public void PrintImageFile(int width, int height)
        {
            PrintDocument printDoc = new PrintDocument();
            printDoc.PrinterSettings.PrinterName = Printer;
            printDoc.PrintController = new StandardPrintController();

            printDoc.DefaultPageSettings.PaperSize = new PaperSize("Custom", width, height);
            printDoc.PrintPage += printDoc_PrintPage;
            printDoc.Print();
        }
        void printDoc_PrintPage(object sender, PrintPageEventArgs e)
        {
            Image imgToPrint = Image.FromFile(File);
            Bitmap myBMP = new Bitmap(imgToPrint);
            myBMP.SetResolution(300, 300);
            e.Graphics.DrawImage(myBMP, 0, 0);
        }
    }
}
